
#####################################
#
# Method adapted from https://www.r-bloggers.com/2018/03/advanced-techniques-with-raster-data-part-3-exercises/
#
#####################################


#Load Data
rm(list=ls())

library(plyr)
library(dplyr)
library(gstat)
library(raster)
library(ggplot2)
library(car)
library(classInt)
library(RStoolbox)
library(caret)
library(caretEnsemble)
library(doParallel)
library(gridExtra)
library(rgdal)

data <- read.csv("/Users/celin/Desktop/IRMA/mergedIRMA.csv", sep=";", dec=",")

data %>%
  dplyr::mutate(across(.cols = c(Species, Ring_number, YEAR, Sampling_site), .fns = as.factor))  %>%
  dplyr::filter(!is.na(Hg_HF_BF_Winter_Spatial) & !is.na(lon_medoids) & lat_medoids >= 40.0) %>%
  dplyr::distinct(Ring_number, YEAR, lon_medoids, lat_medoids, .keep_all = TRUE) ->
  data


# create quick function to stratify (for use in loop)
# I removed the Year bc no variations in Hg and we decided to pool years
stratsample <- function(x){
  x %>%
    splitstackshape::stratified(., group =  c("Species", "Sampling_site", "Ring_number"), size = 1) %>%
    dplyr::arrange(Species, Sampling_site, Ring_number)
}


# create a list of N sub-datasets (each one randomly sampled)
N = 1000
train <- purrr::map(1:N, ~stratsample(data))



# Read shapefil background for map
state<-readOGR(dsn="/Users/celin/Desktop/IRMA/decoup1",layer="decoup1")



#In our case the coordinates columns are already called x and y, so we will use these

#convert this basic data frame into a spatial points data frame
data1<-data
coordinates(data1) = ~lon_medoids+lat_medoids
plot(data1)

x.range <- as.integer(c(-88,71.1))
y.range <- as.integer(c(39, 80))

## now I expand the range of my dataset to a grid with spacing that I want to use in my interpolation

grid <- expand.grid(x=seq(from=x.range[1], to=x.range[2], by=1), y=seq(from=y.range[1], to=y.range[2], by=1))


## convert grid to SpatialPixel class
coordinates(grid) <- ~ x+y
gridded(grid) <- TRUE
par(mfrow = c(1,1))
plot(grid, cex=1.5)
points(data1, pch=1, col='red', cex=1)
title("Interpolation Grid and Sample Points")


#Create dataframes
# train data
train.xy<-list()
train.df<-list()

for (i in 1:1000) {
  train.xy[[i]]<-train[[i]][,c(5:8)] # meaning numeric values being Hg, Longitude and latitude
  train.df[[i]]<-train[[i]][,c(1:8)] # the entire dataset
}

# grid data
grid<-as.data.frame(grid)
grid$sampling_site <- c(NA)
grid$species <- c(NA)
grid$X <- c(NA)
grid$YEAR <- c(NA)
grid$Ring_number <- c(NA)
grid$Hg_HF_BF_Winter_Spatial <- c(NA)

names(grid)<-c("lon_medoids", "lat_medoids")
grid.xy<-grid[,c(1:8)]
grid.df<-grid[,c(1:8)]

names(grid.df)<-c("lon_medoids", "lat_medoids", "Sampling_site", "Species", "X", "YEAR", "Ring_number", "Hg_HF_BF_Winter_Spatial")
names(grid.xy)<-c("lon_medoids", "lat_medoids", "Sampling_site", "Species", "X", "YEAR", "Ring_number", "Hg_HF_BF_Winter_Spatial")

# the one that only contains numeric values
grid.xy1<-list()
for (i in 1:1000) {
  grid.xy1[[i]]<-grid.xy
  grid.xy1[[i]]<-SpatialPointsDataFrame(coords = grid.xy1[[i]][, c("lon_medoids", "lat_medoids")],
                                        data = grid.xy1[[i]])
}

# the one that contains everything
grid.df1<-list()
for (i in 1:1000) {
  grid.df1[[i]]<-grid.df
}


# will contain train.df (so all data)
train.x<-list()

for (i in 1:1000) {
  train.x[[i]]<-train.df[[i]] #
}

#Define coordinates for train.xy in train.xy1, as we need to keep train.xy as it is for later steps. 
train.xy1<-list()

for (i in 1:1000) {
  train.xy1[[i]]<-SpatialPointsDataFrame(coords = train.xy[[i]][, c("lon_medoids", "lat_medoids")],
                                         data = train.xy[[i]])
}

# Run the LMM

lmm.out <- purrr::map(train.x, ~ lmerTest::lmer(log(Hg_HF_BF_Winter_Spatial)~(lon_medoids)+ (lat_medoids)+
                                                  (1|Sampling_site:Species),data=.x, REML=T)) #Here, species is nested inside sampling sites


# check outputs of lmm on random i
i = 10
summary(lmm.out[[i]])
anova(lmm.out[[i]])
print(lmm.out[[1]])


# Only for ICC steps, not needed for the script, but important for the paper stat methodology (see methods)
#library(sjstats)
#library(sjPlot)
#ICC<-list()
#for (i in 1:1000) {
#ICC[[i]]<-sjPlot::tab_model(lmm.out[[i]])
#}

#ICC1<-list()
#for (i in 1:1000){
#ICC1[[i]]<-performance::icc(lmm.out[[i]])
#}

#ICC_adjusted<-list()
#for (i in 1:1000){
#  ICC_adjusted[[i]]<-ICC1[[i]]$ICC_adjusted
#}
#ICC_adjusted1<-do.call(rbind.data.frame, ICC_adjusted)
#summary(ICC_adjusted1)

#quantile(ICC_adjusted1$c.0.67091743263804..0.66471048767262..0.665325789704721..0.669706494361121.., c(0.0250, 0.9750))


# Add the residuals of the LMM  to train.xy1
for (i in 1:1000) {
  train.xy1[[i]]@data<- cbind(train.xy1[[i]]@data, residLMM = resid(lmm.out[[i]]))
}


formMod<-list()
for (i in 1:1000) {
  formMod[[i]] <- residLMM~ 1
}

mod<-list() # store the parameters for the variogram
variog<-list() # store the results of the variogram
variogFitOLS<-list() # store the fitting of the variogram

# Run the variograms for each 1000. Message of error happens for some of the i. This is normal. With such a high number
# of iteration; values for variogram might not fit all the iteration (48 out of 1000 is pretty reasonnable)
for (i in 1:1000) {
  mod[[i]] <- vgm(psill=0.12574091,model  = "Sph", range= 24.54315 , nugget = 0.02123381)
  variog[[i]] <- variogram(formMod[[i]], train.xy1[[i]])
  variogFitOLS[[i]] <- fit.variogram(variog[[i]], model = mod[[i]])
}


# Plot the results on random i 
plot(variog[[10]], variogFitOLS[[10]], main="Semi-variogram of LMM residuals")
plot(variog[[1]], variogFitOLS[[1]], main="Semi-variogram of LMM residuals")

#Predictions_Outputs<-purrr::map(df,~predict(lmm.out[[i]], newdata=.x, type="response"))


# Run the LLM Prediction at grid location
for (i in 1:1000) {
  grid.xy1[[i]]$lmm <- raster::predict(lmm.out[[i]], grid.df1[[i]], re.form=~ 0, type="response")
}

#for (i in 1:10) {
#  grid.xy1[[i]]$lmm <- raster::predict(lmm.out[[i]], grid.df1[[i]], re.form= NA)
#}

#lmm <-purrr::map(grid.df,~raster::predict(lmm.out[[i]], newdata=.x, re.form=~ 0))


#Simple Kriging Prediction of LLM residuals at grid location
SK.LLM<-list()
# Following line is kriging and takes a while to run! Around 1 hour, computer dependent. 
for (i in 1:1000) {
  SK.LLM[[i]] <- krige(formula = formMod[[i]] ,
                       locations = train.xy1[[i]],
                       model = variogFitOLS[[i]],
                       newdata = grid.xy1[[i]])
}


#SK.LLM<-list()
#library(automap)
#for(i in 1:10) {
#  SK.LLM[[i]]<-autoKrige(formula=formMod[[i]],input_data = train.xy1[[i]], new_data = grid.xy1[[i]])
#}

# SK = SImple Kriging
#Kriging prediction (SK + Regression Prediction)
for (i in 1:1000) {
  grid.xy1[[i]]$SK.LLM<-SK.LLM[[i]]$var1.pred
}

# Add RF predicted + SK predicted residuals
for (i in 1:1000) {
  grid.xy1[[i]]$RK.LLM.bc<-(grid.xy1[[i]]$lmm+grid.xy1[[i]]$SK.LLM)
}



# Create a new list to store grid.xy1 data
A<-list()
for (i in 1:1000) {
  A[[i]]<-as.data.frame(grid.xy1[[i]])
}

# Store in LMM_OK the results of LMM + OK
LMM_OK<-list()
for (i in 1:1000) {
  LMM_OK[[i]]<-rasterFromXYZ(A[[i]][, c("lon_medoids", "lat_medoids", "RK.LLM.bc")])
}

# Store the results of LMM only. You don't have too, it is just to test that LMM+OK is better
# than LMM alone or OK alone. 
LMM<-list()
for (i in 1:1000) {
  LMM[[i]]<-rasterFromXYZ(A[[i]][, c("lon_medoids", "lat_medoids", "lmm")])
}

# Store the results of OK only. You don't have too, it is just to test that LMM+OK is better
# than LMM alone or OK alone.
OK<-list()
for (i in 1:1000) {
  OK[[i]]<-rasterFromXYZ(A[[i]][, c("lon_medoids", "lat_medoids", "SK.LLM")])
}

#SK.GLM<-rasterFromXYZ(as.data.frame(grid.xy)[, c("x", "y", "SK.GLM")])
#RK.GLM.bc<-rasterFromXYZ(as.data.frame(grid.xy)[, c("x", "y", "RK.GLM.bc")])
#RK.GLM.SOC<-rasterFromXYZ(as.data.frame(grid.xy)[, c("x", "y", "RK.GLM")])
#Plot predicted SOC

nb.cols<-18
library(RColorBrewer)
pal2 <- colorRampPalette(brewer.pal(8,"RdYlGn"))(nb.cols)
library(RStoolbox)
library(sp)
library(raster)

# Plot LMM alone (choose 1 but you can choose the one you want)
ggR(LMM[[1]], layer="lmm", geom_raster = TRUE) +
  scale_fill_gradientn("", colours = rev(pal2))+
  theme_bw()+
  theme(axis.title.x=element_blank(),
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank(),
        axis.title.y=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank())+
  ggtitle("GLM Predicted")+
  theme(plot.title = element_text(hjust = 0.5))+
  geom_polygon(data = state, aes( x = long, y = lat, group = group), fill="#69b3a2", color="white")


# If you want/need to save the data from LMM, LMM+OK, OK. Useful when you want to store the data
# for later mapping. So you don't have to rerun the kriging that takes time. 

# saveRDS(LMM, "LMM.rds")
# saveRDS(LMM_OK, "LMM_OK.rds")
# saveRDS(OK, "OK.rds")

### You now need to Calculate a Standard Deviation (SD) list depending on the number of iteration
LMM1<-stack(LMM)
LMM_OK1<-stack(LMM_OK)
OK1<-stack(OK)

#create a list to store the results of each SD
sd_essay_LMM1<-list()
sd_essay_LMM_OK1<-list()
sd_essay_OK1<-list()

# calcuate the mean and SD value(as 1000 iteration)
mean_narm = function(x,...){mean(x,na.rm=TRUE)}
sd_narm = function(x,...){sd(x,na.rm=TRUE)}

# Use the following lines to test to be sure that the results from the loop are in accordance 
# with the result for each overlay

#sd_essay0<-overlay(LMM1[[1:1]], fun = sd_narm )
#sd_essay1<-overlay(LMM1[[1:2]], fun = sd_narm )
# sd_essay2<-overlay(C_bis1[[1:3]], fun = sd_narm )
# sd_essay3<-overlay(C_bis1[[1:4]], fun = sd_narm )
# sd_essay4<-overlay(C_bis1[[1:5]], fun = sd_narm )
# sd_essay5<-overlay(C_bis1[[1:6]], fun = sd_narm )


# Following line takes a while to run!
# it will calculate the sd values for LMM, LMM_OK, OK. 
# it is 999 and not 1000 as calculation is made from 1:(i+1)

for (i in 1:999) {
  sd_essay_LMM1[[i]]<-calc(LMM1[[1:(i+1)]], fun = sd_narm)
  sd_essay_LMM_OK1[[i]]<-calc(LMM_OK1[[1:(i+1)]], fun = sd_narm)
  sd_essay_OK1[[i]]<-calc(OK1[[1:(i+1)]], fun = sd_narm)
}

# check the values on random i
sd_essay_LMM1[[9]]
sd_essay_LMM_OK1[[9]]
sd_essay_OK1[[9]]

# plot the values on random i
plot(sd_essay_LMM1[[1]])
plot(sd_essay_LMM_OK1[[1]])
plot(sd_essay_OK1[[1]])


# need to stack the sd_essay to calculate mean sd depending on the number or iterations
# for lmm alone
sd_essayV2_lmm<-stack(sd_essay_LMM1)
mean_sd_essay_LMM1 <- cellStats(sd_essayV2_lmm, "mean") #calculate mean sd for all the rasters, not per cell
plot(mean_sd_essay_LMM1)


# for lmm+ok
sd_essayV2_lmm_ok<-stack(sd_essay_LMM_OK1)
mean_sd_essay_lmm_ok <- cellStats(sd_essayV2_lmm_ok, "mean") #calculate mean sd for all the rasters, not per cell
plot(mean_sd_essay_lmm_ok)


# for ok alone
sd_essayV2_ok<-stack(sd_essay_OK1)
mean_sd_essay_ok <- cellStats(sd_essayV2_ok, "mean") #calculate mean sd for all the rasters, not per cell
plot(mean_sd_essay_ok)

######### Have a mean value from all layers FOR LMM ALONE #########
# Calculate mean
#mean_LMM <- do.call(overlay, c(LMM, fun =mean_narm ))
#plot(mean_LMM)
#saveRDS(mean_LMM, file = "mean_LMM.rds")

#sd_lmm <- do.call(overlay, c(LMM, fun = sd_narm))
#plot(sd_lmm)

#saveRDS(sd_lmm, file = "sd_lmm.rds")

######### Have a mean value from all layers FOR LLL+OK #########
# Calculate mean
mean_lmmok <- do.call(overlay, c(LMM_OK, fun =mean_narm ))
plot(mean_lmmok)
# if you want to save the data so you don't have to rerun it
# saveRDS(mean_lmmok, file = "mean_lmmok.rds")


sd_lmmok <- do.call(overlay, c(LMM_OK, fun = sd_narm))
plot(sd_lmmok)
# if you want to save the data so you don't have to rerun it
# saveRDS(sd_lmmok, file = "sd_lmmok.rds")


######### Have a mean value from all layers FOR OK ALONE #########
# Calculate mean
#mean_OK <- do.call(overlay, c(OK, fun =mean_narm ))
#plot(mean_OK)

# if you want to save the data so you don't have to rerun it
#saveRDS(mean_OK, file = "mean_OK.rds")

#sd_OK <- do.call(overlay, c(OK, fun = sd_narm))
#plot(sd_OK)

# if you want to save the data so you don't have to rerun it
#saveRDS(sd_OK, file = "sd_OK.rds")


# plot as an example for LMM_OK 
ggR(mean_lmmok, geom_raster = TRUE) +
  scale_fill_gradientn("", colours = rev(pal2))+
  theme_bw()+
  theme(axis.title.x=element_blank(),
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank(),
        axis.title.y=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank())+
  ggtitle("LMM + kriging Predicted")+
  theme(plot.title = element_text(hjust = 0.5))+
  geom_polygon(data = state, aes( x = long, y = lat, group = group), fill="#69b3a2", color="white")


#saveRDS(mean_lmmok, file = "hotspot_n1000.rds")


